/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;

/**
 * {@.en BeanDescBag is an wrapper class for BeanDesc to store.It is used for
 * component saving.}
 * 
 * <br />
 * 
 * {@.ja BeanDescBagは複数のBeanDescを保持するためのクラスです．
 * 単一のBeanDescを返す場合、どのような方針で返すかは{@code BeanDescStrategy}に依ります． }
 * 
 * @author Skirnir
 * @author shot
 */
public class BeanDescBag<T> {

	protected Class<T> beanClass;

	protected List<BeanDesc<T>> beanDescList = CollectionsUtil.newArrayList();

	protected BeanDescStrategy beanDescStrategy;

	/**
	 * {@.en Create {@code BeanDescBag} with bean class and {@code
	 * BeanDescStrategy}.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescBagを作成します． }
	 * 
	 * @param beanClass
	 * @param beanDescStrategy
	 */
	public BeanDescBag(Class<T> beanClass, BeanDescStrategy beanDescStrategy) {
		this.beanClass = beanClass;
		this.beanDescStrategy = beanDescStrategy;
	}

	/**
	 * {@.en Add {@code BeanDesc}. Not allowed null.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescを足します．nullは許可されていません．}
	 * 
	 * @param beanDesc
	 */
	public void add(BeanDesc<T> beanDesc) {
		beanDescList.add(Assertion.notNull(beanDesc));
	}

	/**
	 * {@.en Get a {@code BeanDesc}. How to choose a BeanDesc simply depends on
	 * {@code BeanDescStrategy}.}
	 * 
	 * <br />
	 * 
	 * {@.ja
	 * BeanDescを返します．どのように単一のBeanDescを選択するかはBeanDescStrategyに依存しています．}
	 * 
	 * @return
	 */
	public BeanDesc<T> getBeanDesc() {
		return beanDescStrategy.getBeanDesc(beanClass, beanDescList);
	}

	/**
	 * {@.en Get list of {@code BeanDesc}. This method never returns null.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescのリストを返します．このメソッドはnullは返しません．}
	 * 
	 * @return
	 */
	public List<BeanDesc<T>> getBeanDescList() {
		return beanDescList;
	}
}
