/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.lucy.exception.TooManyRegistrationException;

/**
 * {@.en Strategy class for getting BeanDesc or no BeanDesc.}
 * 
 * <br />
 * 
 * {@.ja BeanDescを取得または無いときの振る舞いを定義したクラスです.}
 * 
 * @author shot
 */
public interface BeanDescStrategy {

	/**
	 * {@.en Get {@link BeanDesc} from the {@link Class} and {@link List
	 * <BeanDesc<T>>} . By default, return {@link BeanDesc} if there is one
	 * component in {@link List}.If more than one, throw
	 * {@link TooManyRegistrationException}.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescを与えられたClassとBeanDescのListから取得します．
	 * デフォルトの挙動では、List内に一つしかBeanDescが無い場合、それを返します．複数ある場合、{@code
	 * TooManyRegistrationException}が発生します．}
	 * 
	 * @param <T>
	 * @param beanClass
	 * @param beanDescList
	 * @throws TooManyRegistrationException
	 * @return
	 * @see org.t2framework.lucy.impl.DefaultBeanDescStrategyImpl
	 * @see org.t2framework.lucy.exception.TooManyRegistrationException
	 */
	<T> BeanDesc<T> getBeanDesc(Class<T> beanClass,
			List<BeanDesc<T>> beanDescList) throws TooManyRegistrationException;

	/**
	 * {@.en Handle if there is no {@link BeanDesc}.By default, just return
	 * null.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescが存在しない場合の挙動を決めるメソッドです．デフォルトでは、nullを返します．}
	 * 
	 * @param <T>
	 * @param key
	 * @return
	 * @see org.t2framework.lucy.impl.DefaultBeanDescStrategyImpl
	 */
	<T> T handleNoBeanDesc(Object key);
}
