/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import java.util.List;
import java.util.Map;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.DescType;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.spi.Behavior;

/**
 * 
 * {@.en ConfigBindResult is a stored result of component. }
 * 
 * <br />
 * 
 * {@.ja {@link ConfigBindResult}はコンポーネントの情報や振る舞いのバインド結果を格納します．}
 * 
 * @author shot
 * 
 */
public class ConfigBindResult {

	protected BeanDesc<?> beanDesc;

	protected boolean initialized = false;

	protected Map<DescType, List<Behavior>> map = CollectionsUtil
			.newLinkedHashMap();

	protected List<Behavior> allBehaviors = CollectionsUtil.newLinkedList();

	public ConfigBindResult(BeanDesc<?> beanDesc) {
		this.beanDesc = beanDesc;
	}

	public BeanDesc<?> getKey() {
		return beanDesc;
	}

	public void addBehavior(DescType desc, Behavior b) {
		Assertion.notNull(desc);
		Assertion.notNull(b);
		List<Behavior> list = map.get(desc);
		if (list == null) {
			list = CollectionsUtil.newArrayList();
		}
		list.add(b);
		map.put(desc, list);
		allBehaviors.add(b);
	}

	public List<Behavior> getBehaviorList(DescType desc) {
		return map.get(desc);
	}

	public boolean hasClassDesc(ClassDesc<?> cd) {
		return cd != null && map.containsKey(cd);
	}

	public boolean hasMethodDesc(MethodDesc md) {
		return md != null && map.containsKey(md);
	}

	public boolean hasPropertyDesc(PropertyDesc<?> pd) {
		return pd != null && map.containsKey(pd);
	}

	public void initializeBehaviors() {
		for (Behavior b : allBehaviors) {
			b.init();
		}
		initialized = true;
	}

	public void destroyBehaviors() {
		for (Behavior b : allBehaviors) {
			b.destroy();
		}
	}

	public boolean isInitialized() {
		return initialized;
	}

	public boolean isEmpty() {
		return map.isEmpty();
	}
}
