/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.lucy.config.meta.InstanceConfig;
import org.t2framework.lucy.spi.Behavior;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * {@link ConfigBinder} is the interface to bind component and behavior.
 * 
 * @author shot
 */
public interface ConfigBinder {

	/**
	 * Add {@link Behavior}.
	 * 
	 * @param behavior
	 */
	void addBehavior(Class<? extends Behavior> behavior);

	/**
	 * Add {@link InstanceConfig}.
	 * 
	 * @param instanceConfig
	 */
	void addInstanceConfig(InstanceConfig instanceConfig);

	/**
	 * Bind {@link BeanDesc} and {@link Behavior}.This result is as
	 * {@link ConfigBindResult}.
	 * 
	 * @param <T>
	 * @param beanDesc
	 * @return
	 */
	<T> ConfigBindResult bind(BeanDesc<T> beanDesc);

	/**
	 * Load configuration.
	 * 
	 * @param conf
	 * @return
	 */
	ConfigBinder loadConfiguration(final LucyConfiguration conf);
}
