/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

/**
 * 
 * {@.en Lifecycle of {@link Lucy} itself and components.}
 * 
 * <br />
 * 
 * {@.ja {@link Lucy}とコンポーネントのライフサイクルを規定するenumです.}
 * 
 * @author shot
 * 
 */
public enum Lifecycle {

	/**
	 * Container itself is now created.
	 */
	CONTAINER_CREATED,

	/**
	 * {@.en Container configuration phase.}
	 * 
	 * <br />
	 * 
	 * {@.ja {@link Lucy}自身の設定フェーズです.}
	 */
	CONTAINER_CONFIGURED,

	/**
	 * Class loading phase.
	 */
	CLASS_LOADING,

	/**
	 * Component creating phase.
	 */
	COMPONENT_CREATING,

	/**
	 * The phase just after components are created.
	 */
	COMPONENT_CREATED,

	/**
	 * Component initializing phase.
	 */
	COMPONENT_INITIALIZE,

	/**
	 * Component destroy phase.
	 */
	COMPONENT_DESTROY,

	/**
	 * Container shutdown.
	 */
	CONTAINER_DESTROY;

	/**
	 * Get component creation lifecycle.
	 */
	public static Lifecycle[] getComponentLifecycle() {
		return new Lifecycle[] { CLASS_LOADING, COMPONENT_CREATING,
				COMPONENT_CREATED, COMPONENT_INITIALIZE, COMPONENT_DESTROY };
	}

}
