/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.annotation.core;

import static java.lang.annotation.ElementType.CONSTRUCTOR;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * 
 * {@.en &#064;Inject annotation is the annotation to inject component.<br />
 * 
 * Lucy supports method injection, setter injection, and constructor injection.
 * * }
 * 
 * <br />
 * 
 * {@.ja &#064;Injectアノテーションはコンポーネントへのインジェクトポイントを明記するためのアノテーションです
 * 
 * {@code Lucy}はメソッドインジェクション、セッターインジェクション、コンストラクタインジェクションをサポートしています．}
 * 
 * @author shot
 * 
 */
@Retention(RUNTIME)
@Target( { METHOD, CONSTRUCTOR })
public @interface Inject {

	String TYPES = "types";

	String DEFAUL_NAME = "";

	int DEFAULT_NAME_LENGTH = 1;

	/**
	 * 
	 * {@.en types() are used when inject multiple component to one method.}
	 * 
	 * <br />
	 * 
	 * {@.ja 複数コンポーネントインジェクションするときに、型補完するためのメソッドです．}
	 * 
	 * @return
	 */
	Class<?>[] types() default Object.class;

	/**
	 * 
	 * {@.en Inject by name.}
	 * 
	 * <br />
	 * 
	 * {@.ja 名前によるインジェクションです．}
	 * 
	 * @return
	 */
	String[] name() default "";
}
