/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop;

import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.spi.Aspect;

/**
 * Aspect descriptor interface.It has original class, enhanced class, name,
 * aspects.
 * 
 * @author shot
 */
public interface AspectDesc {

	/**
	 * Get original class.
	 * 
	 * @return
	 */
	Class<?> getOriginalClass();

	/**
	 * Get enhanced class.
	 * 
	 * @return
	 */
	Class<?> getEnhancedClass();

	/**
	 * Get enhanced class name.
	 * 
	 * @return
	 */
	String getEnhancedClassName();

	/**
	 * Set enhanced class.
	 * 
	 * @param enhancedClass
	 */
	void setEnhancedClass(Class<?> enhancedClass);

	/**
	 * Set enhanced class name.
	 * 
	 * @param name
	 */
	void setEnhancedClassName(String name);

	/**
	 * Get list of {@link Aspect}.
	 * 
	 * @return
	 */
	List<Aspect> getAspects();

	/**
	 * Get parameters.
	 * 
	 * @return
	 */
	Map<String, Object> getParameters();

	/**
	 * If target is interface or not.
	 * 
	 * @return
	 */
	boolean isInterfaceTarget();

}
