/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop;

/**
 * {@link EnhancedClassCache} is simple cache interface of enhanced class.
 * 
 * @author shot
 * 
 */
public interface EnhancedClassCache {

	/**
	 * Get enhanced class from cache.
	 * 
	 * @param <T>
	 * @param originalClass
	 * @return
	 */
	<T> Class<? extends T> get(Class<? extends T> originalClass);

	/**
	 * Cache enhanced class.The original class is the key of cache.
	 * 
	 * @param <T>
	 * @param originalClass
	 * @param enhancedClass
	 */
	<T> void cache(Class<? extends T> originalClass,
			Class<? extends T> enhancedClass);

	/**
	 * Clear cache for the original class.
	 * 
	 * @param <T>
	 * @param originalClass
	 */
	<T> void clear(Class<? extends T> originalClass);

	/**
	 * Clear all cache.
	 */
	void clear();
}
