/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.config.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Pointcut;
import org.t2framework.commons.el.Expression;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Logger;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.aop.config.AspectConfig;
import org.t2framework.lucy.aop.impl.AspectImpl;

/**
 * 
 * {@.en {@link AspectConfig} implementation.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public class AspectConfigImpl extends AbstractAspectConfig {

	protected static Logger logger = Logger.getLogger(AspectConfigImpl.class);

	protected Lucy lucy;

	protected Class<?> targetClass;

	protected Pointcut pointcut;

	public AspectConfigImpl(Lucy lucy, Pointcut pointcut) {
		this.lucy = Assertion.notNull(lucy);
		this.pointcut = Assertion.notNull(pointcut);
	}

	@Override
	public Pointcut getPointcut() {
		return pointcut;
	}

	@Override
	public Class<?> getTargetClass() {
		return targetClass;
	}

	@Override
	public void setPointcut(Pointcut pointcut) {
		this.pointcut = Assertion.notNull(pointcut);
	}

	@Override
	public void setTargetClass(Class<?> targetClass) {
		this.targetClass = Assertion.notNull(targetClass);
	}

	@Override
	public Aspect createAspect() {
		List<Interceptor> list = findInterceptors();
		Aspect aspect = new AspectImpl(list);
		aspect.setPointcut(pointcut);
		return aspect;
	}

	@SuppressWarnings("unchecked")
	protected List<Interceptor> findInterceptors() {
		List<Interceptor> list = CollectionsUtil.newArrayList();
		for (Expression e : getExpressions()) {
			Object result = e.evaluate(lucy);
			if (result instanceof Interceptor) {
				list.add((Interceptor) result);
			} else if (result instanceof Collection) {
				Collection c = (Collection) result;
				add(list, c);
			} else if (result != null) {
				Class<?> c = result.getClass();
				if (c.isArray()) {
					Object[] ary = (Object[]) result;
					add(list, Arrays.asList(ary));
				} else {
					logger.log("WLucyAop0001", new Object[] { result });
				}
			} else {
				logger.log("WLucyAop0001", new Object[] { result });
			}
		}
		return list;
	}

	protected void add(List<Interceptor> list, Collection<?> maybe) {
		for (Object o : maybe) {
			if (o instanceof Interceptor) {
				Interceptor i = (Interceptor) o;
				list.add(i);
			} else {
				logger.log("WLucyAop0002", new Object[] { o,
						(o != null) ? o.getClass() : null });
			}
		}
	}
}
