/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.impl;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Pointcut;
import org.t2framework.commons.util.Assertion;

public class AspectImpl implements Aspect, Serializable {

	static final long serialVersionUID = 0L;

	protected List<Interceptor> interceptors;

	protected Pointcut pointcut;

	public AspectImpl(Interceptor... interceptors) {
		this(Arrays.asList(interceptors));
	}

	public AspectImpl(String regex, Interceptor... interceptors) {
		this(regex, Arrays.asList(interceptors));
	}

	public AspectImpl(List<Interceptor> interceptors) {
		this(Pointcut.DEFAULT_REGEX, Assertion.notNull(interceptors));
	}

	public AspectImpl(String regex, List<Interceptor> interceptors) {
		this.interceptors = Assertion.notNull(interceptors);
		this.pointcut = new PointcutImpl(Assertion.notNull(regex));
	}

	@Override
	public Pointcut getPointcut() {
		return pointcut;
	}

	@Override
	public void setPointcut(Pointcut pointcut) {
		this.pointcut = Assertion.notNull(pointcut);
	}

	@Override
	public List<Interceptor> getInterceptors() {
		return interceptors;
	}

}
