/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.javassist;

import java.util.List;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.EnhancedClassBuilder;
import org.t2framework.lucy.aop.InterceptorDescStrategy;
import org.t2framework.lucy.aop.impl.AbstractAopEngine;
import org.t2framework.lucy.aop.impl.AspectDescImpl;
import org.t2framework.lucy.aop.util.AopUtil;

/**
 * JavassistAopEngineImpl is implementation for Javassist byte code enhance AOP.
 * 
 * @author shot
 * @author c9katayama
 */
public class JavassistAopEngineImpl extends AbstractAopEngine {

	public JavassistAopEngineImpl() {
	}

	public JavassistAopEngineImpl(InterceptorDescStrategy strategy) {
		super(strategy);
	}

	@Override
	protected <T> AspectDesc createAspectDesc(Class<T> originalClass,
			List<Aspect> aspects) {
		return new AspectDescImpl(originalClass, AopUtil
				.createEnhancedClassName(originalClass), aspects);
	}

	@Override
	protected <T> EnhancedClassBuilder<T> createBuilder(
			InterceptorDescStrategy strategy) {
		return new JavassistEnhancedClassBuilderImpl<T>(strategy);
	}
}
