/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.module;

import org.t2framework.commons.aop.AopEngine;
import org.t2framework.commons.meta.spi.AnnotationConfigSupport;
import org.t2framework.commons.util.Reflections.ClassUtil;
import org.t2framework.lucy.aop.util.AopUtil;
import org.t2framework.lucy.config.stax.XmlEventHandler;
import org.t2framework.lucy.module.LucyAopModule;
import org.t2framework.lucy.spi.Behavior;

/**
 * Select proxy-based AOP module or Javassist-based AOP module.
 * 
 * @author c9katayama
 * 
 */
public class SelectableLucyAopModuleImpl implements LucyAopModule {

	protected LucyAopModule selectModule;

	public SelectableLucyAopModuleImpl() {
		selectModule();
	}

	protected void selectModule() {
		if (AopUtil.isJavassistEnabled()) {
			selectModule = ClassUtil
					.newInstanceByName("org.t2framework.lucy.aop.javassist.module.JavassistAopModuleImpl");
		} else {
			selectModule = ClassUtil
					.newInstanceByName("org.t2framework.lucy.aop.proxy.module.ProxyAopModuleImpl");
		}
	}

	@Override
	public XmlEventHandler getAspectTagHandler() {
		return selectModule.getAspectTagHandler();
	}

	@Override
	public <S> S getService(Class<? super S> serviceClass) {
		return (S) selectModule.getService(serviceClass);
	}

	@Override
	public Class<? extends Behavior> getAspectBehaviorClass() {
		return selectModule.getAspectBehaviorClass();
	}

	@Override
	public AnnotationConfigSupport getAnnotationConfigSupport() {
		return selectModule.getAnnotationConfigSupport();
	}

	@Override
	public AopEngine getAopEngine() {
		return selectModule.getAopEngine();
	}
}
