/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.proxy;

import java.lang.reflect.Proxy;

import org.t2framework.commons.util.Reflections.ClassLoaderUtil;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.EnhancedClassBuilder;
import org.t2framework.lucy.aop.InterceptorDescStrategy;
import org.t2framework.lucy.aop.exception.AspectException;

/**
 * Build proxy class.
 * 
 * @author AKatayama
 * 
 * @param <T>
 */
public class ProxyEnhancedClassBuilderImpl<T> implements
		EnhancedClassBuilder<T> {

	public void setInterceptorDescStrategy(InterceptorDescStrategy strategy) {
		// no op;
	};

	/**
	 * Build an enhanced class.
	 */
	@Override
	@SuppressWarnings("unchecked")
	public Class<T> build(AspectDesc aspectDesc) {
		Class<?> targetClass = aspectDesc.getOriginalClass();
		Class<?>[] proxyInterfaces;
		if (targetClass.isInterface()) {
			proxyInterfaces = new Class[] { targetClass };
		} else {
			proxyInterfaces = targetClass.getInterfaces();
			if (proxyInterfaces == null) {
				// TODO
				throw new AspectException(new IllegalArgumentException(
						"Cannot create proxy instance."));
			}
		}
		return (Class<T>) Proxy.getProxyClass(ClassLoaderUtil
				.getClassLoader(targetClass), proxyInterfaces);
	}
}
