/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.proxy;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Reflections.MethodUtil;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.InterceptorDesc;
import org.t2framework.lucy.aop.InterceptorDescStrategy;
import org.t2framework.lucy.aop.impl.InterceptorDescStrategyImpl;

/**
 * @author c9katayama
 */
public class ProxyInterceptorDescStrategyImpl extends
		InterceptorDescStrategyImpl implements InterceptorDescStrategy {

	@Override
	protected <T> void assemble0(Map<AccessibleObject, InterceptorDesc> map,
			MethodDesc methodDesc, AspectDesc aspectDesc) {
		final List<Interceptor> list = CollectionsUtil.newArrayList();
		if (methodDesc == null) {
			return;
		}
		for (Aspect a : aspectDesc.getAspects()) {
			LOG.log("DLucyAop0005", new Object[] {
					methodDesc.getDeclaringClass().getName(),
					methodDesc.getMethodName(),
					a.getPointcut().getPatternString() });
			if (a.getPointcut().isApplied(methodDesc.getMethod())) {
				list.addAll(a.getInterceptors());
				LOG.log("DLucyAop0006", new Object[] {
						methodDesc.getDeclaringClass().getName(),
						methodDesc.getMethodName() });
			} else {
				LOG.log("DLucyAop0007", new Object[] {
						methodDesc.getDeclaringClass().getName(),
						methodDesc.getMethodName() });
			}
		}
		if (list.size() == 0) {
			return;
		}
		Method targetMethod = methodDesc.getMethod();
		Class<?>[] interfaces = aspectDesc.getEnhancedClass().getInterfaces();
		for (int i = 0; i < interfaces.length; i++) {
			Method interfaceMethod = MethodUtil.getMethodNoException(
					interfaces[i], targetMethod.getName(), targetMethod
							.getParameterTypes());
			if (interfaceMethod != null) {
				map.put(interfaceMethod,
						createInterceptorDesc(methodDesc, list));
			}
		}
	}
}
