/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.aop.proxy;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Invocation;
import org.t2framework.lucy.aop.InterceptorDesc;
import org.t2framework.lucy.aop.impl.InvocationImpl;

/**
 * Proxy AOP invocation handler.
 * 
 * @author c9katayama
 */
public class ProxyInvocationHandler implements InvocationHandler {

	protected Map<AccessibleObject, InterceptorDesc> proxyInterceptorMap;

	protected Object thisInstance;

	public ProxyInvocationHandler(
			Map<AccessibleObject, InterceptorDesc> proxyInterceptorMap,
			Object thisInstance) {
		this.proxyInterceptorMap = proxyInterceptorMap;
		this.thisInstance = thisInstance;
	}

	@Override
	public Object invoke(Object proxyInstance, Method method, Object[] args)
			throws Throwable {

		InterceptorDesc interceptorDesc = proxyInterceptorMap.get(method);
		if (interceptorDesc != null) {
			List<Interceptor> interceptorList = interceptorDesc
					.getInterceptors();
			if (interceptorList != null && interceptorList.size() > 0) {
				Invocation<? extends AccessibleObject> invocation = new InvocationImpl(
						interceptorList, method, method, args, proxyInstance
								.getClass(), thisInstance);
				Object result = invocation.proceed();
				return result;
			}
		}
		Object result = method.invoke(thisInstance, args);
		return result;
	}
}
