/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.behavior;

import java.util.Map;

import org.t2framework.commons.Constants;
import org.t2framework.commons.Disposable;
import org.t2framework.commons.Disposer;
import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.annotation.core.DestroyMethod;
import org.t2framework.lucy.config.meta.impl.DestroyMethodConfig;
import org.t2framework.lucy.config.meta.impl.LifeCycleMethodConfig;
import org.t2framework.lucy.spi.AbstractBehavior;

/**
 * DestroyMethodBehavior is a behavior class.It will invoke target method for
 * {@link DestroyMethod} as annotation configuration, and
 * {@link DestroyMethodConfig} as xml configuration.
 * 
 * @author taichi
 */
@ConfigurationTarget( { DestroyMethod.class, DestroyMethodConfig.class })
public class DestroyMethodBehavior extends AbstractBehavior {

	protected Map<MethodDesc, Object> destroyedCache = CollectionsUtil
			.newHashMap();

	@SuppressWarnings("unchecked")
	public DestroyMethodBehavior(BeanDesc beanDesc, MethodDesc md, Config cd) {
		super(beanDesc, md, cd);
	}

	@Override
	public <T> T execute(final T t, Lucy lucy) {
		final MethodDesc methodDesc = getMethodDesc();
		if (methodDesc != null) {
			if (this.destroyedCache.containsKey(methodDesc) == false) {
				Disposer.add(new Disposable() {
					@Override
					public void dispose() {
						Config cd = getConfig();
						if (cd == null || cd.hasAnnotation()) {
							methodDesc.invoke(t, Constants.EMPTY_ARRAY);
						} else {
							methodDesc.invoke(t, ((LifeCycleMethodConfig) cd)
									.getArgs());
						}
						DestroyMethodBehavior.this.destroyedCache.put(
								methodDesc, t);
					}
				});
			}
		}
		return t;
	}

	@Override
	public <T> void destroy() {
		this.destroyedCache.clear();
	}

	@Override
	public Lifecycle getLifecycle() {
		return Lifecycle.COMPONENT_DESTROY;
	}
}
