/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.behavior;

import java.util.Map;

import org.t2framework.commons.Constants;
import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.annotation.core.InitMethod;
import org.t2framework.lucy.config.meta.impl.InitMethodConfig;
import org.t2framework.lucy.config.meta.impl.LifeCycleMethodConfig;
import org.t2framework.lucy.spi.AbstractBehavior;
import org.t2framework.lucy.spi.Behavior;

/**
 * 
 * {@.en InitMethodBehavior is a behavior class for invoking method as
 * initialize method.The target configurations are {@link InitMethod} as
 * annotation configuration, and {@link InitMethodConfig} as xml configuration.
 * }
 * 
 * <br />
 * 
 * {@.ja {@link InitMethodBehavior}は初期化メソッドを実行する{@link Behavior}です．}
 * 
 * @author shot
 * 
 */
@ConfigurationTarget( { InitMethod.class, InitMethodConfig.class })
public class InitMethodBehavior extends AbstractBehavior {

	protected Map<MethodDesc, Object> initedCache = CollectionsUtil
			.newHashMap();

	@SuppressWarnings("unchecked")
	public InitMethodBehavior(BeanDesc beanDesc, MethodDesc md, Config cd) {
		super(beanDesc, md, cd);
	}

	/**
	 * Execute initialize method invoking.
	 */
	@Override
	public <T> T execute(T t, Lucy lucy) {
		MethodDesc methodDesc = getMethodDesc();
		if (methodDesc != null) {
			if (!this.initedCache.containsKey(methodDesc)) {
				Config cd = getConfig();
				if (cd == null || cd.hasAnnotation()) {
					methodDesc.invoke(t, Constants.EMPTY_ARRAY);
				} else {
					methodDesc
							.invoke(t, ((LifeCycleMethodConfig) cd).getArgs());
				}
				this.initedCache.put(methodDesc, t);
			}
		}
		return t;
	}

	@Override
	public <T> void destroy() {
		this.initedCache.clear();
	}

	@Override
	public Lifecycle getLifecycle() {
		return Lifecycle.COMPONENT_INITIALIZE;
	}
}
