/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.behavior;

import java.util.Properties;

import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.annotation.core.PropertiesInject;
import org.t2framework.lucy.spi.AbstractBehavior;

/**
 * Java properties file injection behavior.
 * 
 * @author shot
 * 
 */
@ConfigurationTarget(PropertiesInject.class)
public class PropertiesInjectBehavior extends AbstractBehavior {

	@SuppressWarnings("unchecked")
	public PropertiesInjectBehavior(BeanDesc beanDesc, MethodDesc md, Config cd) {
		super(beanDesc, md, cd);
	}

	@SuppressWarnings("unchecked")
	public PropertiesInjectBehavior(BeanDesc beanDesc, PropertyDesc pd,
			Config cd) {
		super(beanDesc, pd, cd);
	}

	@Override
	public <T> T execute(T t, Lucy lucy) {
		Assertion.notNull(lucy);
		String value = null;
		PropertiesInject propertiesInject = (PropertiesInject) getConfig()
				.getAnnotation();
		String propertiesFileName = propertiesInject.name();
		String key = propertiesInject.key();
		value = findValue(propertiesFileName, key, lucy);
		if (value != null) {
			invoke(t, new Object[] { value });
		}
		return t;
	}

	protected <T> void invoke(T t, Object[] args) {
		MethodDesc methodDesc = getMethodDesc();
		if (methodDesc == null) {
			methodDesc = getPropertyDesc().getWriteMethodDesc();
		}
		methodDesc.invoke(t, args);
	}

	protected String findValue(final String propertiesFileName,
			final String key, final Lucy lucy) {
		if (lucy.hasComponent(propertiesFileName)) {
			Properties prop = lucy.get(propertiesFileName);
			return prop.getProperty(key);
		} else {
			Properties[] all = lucy.getAll(Properties.class);
			for (Properties p : all) {
				if (p.containsKey(key)) {
					return p.getProperty(key);
				}
			}
		}
		return null;
	}

	@Override
	public Lifecycle getLifecycle() {
		return Lifecycle.COMPONENT_CREATED;
	}

}
