/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.meta;

import java.util.Map;

import org.t2framework.commons.exception.NoSuchInstaceConfigRuntimeException;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.config.meta.impl.PrototypeInstanceConfig;
import org.t2framework.lucy.config.meta.impl.SingletonInstanceConfig;

/**
 * 
 * {@.en Instance creating factory. }
 * 
 * <br />
 * 
 * {@.ja インスタンス生成するための{@link InstanceConfig}を取得するためのファクトリです．}
 * 
 * @author shot
 * 
 */
public class InstanceConfigFactory {

	public static final String SINGLETON = "singleton";

	public static final String PROTOTYPE = "prototype";

	public static final String DEFAULT = SINGLETON;

	protected static final Map<String, InstanceConfig> map = CollectionsUtil
			.newHashMap();

	static {
		map.put(SINGLETON, new SingletonInstanceConfig());
		map.put(PROTOTYPE, new PrototypeInstanceConfig());
	}

	public static InstanceConfig getInstanceConfig(String type) {
		Assertion.notNull(type);
		InstanceConfig instanceConfig = map.get(type);
		if (instanceConfig == null) {
			throw new NoSuchInstaceConfigRuntimeException(type);
		}
		return instanceConfig;
	}

	public static void addInstanceConfig(InstanceConfig instanceConfig) {
		Assertion.notNull(instanceConfig);
		map.put(instanceConfig.getName(), instanceConfig);
	}

}
