/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import java.util.Iterator;
import java.util.Map;

import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.StartElement;

import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.PrintableUtil;

/**
 * 
 * {@.en Attributes for StAX parsing.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public class Attributes {

	protected Map<String, Attribute> map = CollectionsUtil.newHashMap();

	@SuppressWarnings("unchecked")
	public Attributes(StartElement se) {
		Assertion.notNull(se);
		init(se.getAttributes());
	}

	public Attributes(Map<String, Attribute> map) {
		this.map = Assertion.notNull(map);
	}

	protected void init(Iterator<Attribute> iterator) {
		for (; iterator.hasNext();) {
			Attribute attribute = iterator.next();
			map.put(attribute.getName().getLocalPart(), attribute);
		}
	}

	public boolean hasAnyAttribute() {
		return map.size() > 0;
	}

	public boolean hasAttribute(String name) {
		return map.containsKey(name);
	}

	public String getValue(String name) {
		Attribute attribute = map.get(name);
		return (attribute != null) ? attribute.getValue() : null;
	}

	public String getAndRemoveValue(String name) {
		Attribute attribute = map.remove(name);
		return (attribute != null) ? attribute.getValue() : null;
	}

	public boolean isSpecified(String name) {
		Attribute attribute = map.get(name);
		return (attribute != null) ? attribute.isSpecified() : false;
	}

	public String getAllValueAsString() {
		return PrintableUtil.toPrintableString(map.values());
	}

}
