/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.NameDesc;
import org.t2framework.commons.meta.impl.ClassDescImpl;
import org.t2framework.commons.meta.impl.NameDescImpl;
import org.t2framework.commons.util.MessageFormatter;
import org.t2framework.commons.util.StringUtil;
import org.t2framework.commons.util.Reflections.ClassUtil;
import org.t2framework.lucy.config.meta.AutoInject;
import org.t2framework.lucy.config.meta.ConfigurableBeanDesc;
import org.t2framework.lucy.config.meta.InstanceConfigFactory;
import org.t2framework.lucy.config.stax.LucyConfigTagHandler.LucyConfig;

/**
 * 
 * {@.en Tag handler for <component>.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public class ComponentTagHandler implements XmlEventHandler {

	public static final String CLASS_ATTRIBUTE = "class";

	public static final String INSTANCE_ATTRIBUTE = "instance";

	public static final String NAME_ATTRIBUTE = "name";

	public static final String AUTOINJECT_ATTRIBUTE = "autoinject";

	@Override
	public void start(XmlEventContext context, Attributes attributes) {
		String value = attributes.getAndRemoveValue(CLASS_ATTRIBUTE);
		if (StringUtil.isEmpty(value)) {
			final String message = MessageFormatter.getMessage("ELucyCore0014",
					new Object[] { "component", context.getLineNumber(),
							context.getColumnNumber(), "class" });
			throw new IllegalStateException(message);
		}
		final Class<?> c = ClassUtil.forName(value);
		String instance = attributes.getAndRemoveValue(INSTANCE_ATTRIBUTE);
		if (StringUtil.isEmpty(instance)) {
			instance = InstanceConfigFactory.DEFAULT;
		}
		ConfigurableBeanDesc<Object> beanDesc = createConfigurableBeanDesc(c,
				instance);
		String name = attributes.getAndRemoveValue(NAME_ATTRIBUTE);
		if (StringUtil.isEmpty(name) == false) {
			beanDesc.setNameDesc(createNameDesc(name));
		}

		String autoinject = attributes.getAndRemoveValue(AUTOINJECT_ATTRIBUTE);
		beanDesc.setAutoinject(autoinject);

		// check illegal attribute.

		context.push(beanDesc);
	}

	@Override
	public void end(XmlEventContext context, String body) {
		ConfigurableBeanDesc<?> beanDesc = (ConfigurableBeanDesc<?>) context
				.pop();
		AutoInject.get(beanDesc.getAutoinject()).setupAutoInject(beanDesc);
		LucyConfig config = (LucyConfig) context.peek();
		config.addBeanDesc(beanDesc);
	}

	protected ConfigurableBeanDesc<Object> createConfigurableBeanDesc(
			Class<?> c, String instance) {
		return new ConfigurableBeanDesc<Object>(createClassDesc(c, instance));
	}

	protected ClassDesc<Object> createClassDesc(Class<?> clazz, String instance) {
		ClassDesc<Object> cd = new ClassDescImpl<Object>(clazz);
		cd.addConfig(InstanceConfigFactory.getInstanceConfig(instance));
		return cd;
	}

	protected NameDesc<Object> createNameDesc(String name) {
		return new NameDescImpl<Object>(name);
	}

}
