/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.XMLEvent;

import org.t2framework.commons.exception.XMLStreamRuntimeException;
import org.t2framework.commons.util.Assertion;

/**
 * 
 * {@.en Delegation class for {@link XMLEventReader}.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public class XMLEventReaderDelegate implements XMLEventReader {

	protected final XMLEventReader original;

	protected XMLEvent before = null;

	public XMLEventReaderDelegate(XMLEventReader original) {
		this.original = Assertion.notNull(original);
	}

	public void close() throws XMLStreamRuntimeException {
		try {
			original.close();
		} catch (XMLStreamException e) {
			throw new XMLStreamRuntimeException(e);
		}
	}

	public String getElementText() throws XMLStreamRuntimeException {
		try {
			return original.getElementText();
		} catch (XMLStreamException e) {
			throw new XMLStreamRuntimeException(e);
		}
	}

	public Object getProperty(String name) throws IllegalArgumentException {
		return original.getProperty(name);
	}

	public boolean hasNext() {
		return original.hasNext();
	}

	public Object next() {
		Object next = original.next();
		return next;
	}

	public XMLEvent nextEvent() throws XMLStreamRuntimeException {
		try {
			return original.nextEvent();
		} catch (XMLStreamException e) {
			throw new XMLStreamRuntimeException(e);
		}
	}

	public XMLEvent nextTag() throws XMLStreamRuntimeException {
		try {
			return original.nextTag();
		} catch (XMLStreamException e) {
			throw new XMLStreamRuntimeException(e);
		}
	}

	public XMLEvent peek() throws XMLStreamRuntimeException {
		try {
			return original.peek();
		} catch (XMLStreamException e) {
			throw new XMLStreamRuntimeException(e);
		}
	}

	public void remove() {
		original.remove();
	}

}
