/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.module;

import org.t2framework.commons.aop.AopEngine;
import org.t2framework.commons.meta.spi.AnnotationConfigSupport;
import org.t2framework.commons.module.Module;
import org.t2framework.lucy.config.stax.XmlEventHandler;
import org.t2framework.lucy.spi.Behavior;

/**
 * 
 * {@.en }
 * 
 * <br />
 * 
 * {@.ja LucyのAOPモジュールインタフェースです.AOPに必要なモジュールを提供します.}
 * 
 * @author shot
 * 
 */
public interface LucyAopModule extends Module {

	/**
	 * 
	 * {@.en Get aspect tag handler.}
	 * 
	 * <br />
	 * 
	 * {@.ja aspectタグのハンドラを返します．}
	 * 
	 * @return
	 */
	XmlEventHandler getAspectTagHandler();

	/**
	 * 
	 * {@.en Get class of aspect {@link Behavior}.}
	 * 
	 * <br />
	 * 
	 * {@.ja アスペクトエンハンス用の{@link Behavior}を返します.}
	 * 
	 * @return
	 */
	Class<? extends Behavior> getAspectBehaviorClass();

	/**
	 * 
	 * {@.en Get {@link AnnotationConfigSupport}, which is a helper class for
	 * annotation handling.}
	 * 
	 * <br />
	 * 
	 * {@.ja アノテーションのヘルパクラスである{@link AnnotationConfigSupport}を返します．}
	 * 
	 * @return
	 */
	AnnotationConfigSupport getAnnotationConfigSupport();

	/**
	 * 
	 * {@.en Get {@link AopEngine}, an engine class for aspect weaving.}
	 * 
	 * <br />
	 * 
	 * {@.ja AOPのコアエンジンの{@link AopEngine}を返します．}
	 * 
	 * @return
	 */
	AopEngine getAopEngine();
}
