/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.spi;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.lucy.BeanDescStrategy;
import org.t2framework.lucy.ConfigBinder;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyComponentRegistrar;

/**
 * Configuration class for {@link Lucy}.
 * 
 * @author shot
 */
public interface LucyConfiguration {

	/**
	 * Load {@link ConfigBinder}, the class for mapping annotation/xml
	 * configuration and {@link Behavior}.
	 * 
	 * @return
	 */
	ConfigBinder loadConfigBinder();

	/**
	 * Create {@link ComponentBuilder}.
	 * 
	 * @param beanDesc
	 * @param lucy
	 * @return
	 */
	ComponentBuilder createBuilder(BeanDesc<?> beanDesc, Lucy lucy);

	/**
	 * Get {@link BeanDescStrategy} as configurable strategy class for
	 * {@link BeanDesc}.
	 * 
	 * @param lucy
	 * @return
	 */
	BeanDescStrategy getBeanDescStrategy(Lucy lucy);

	/**
	 * Get {@link LucyComponentRegistrar} from the given resource as a key.
	 * 
	 * @param <R>
	 * @param resource
	 * @return
	 */
	<R> LucyComponentRegistrar<R> getAcceptableComponentRegistrar(R resource);
}
